from piscript.DviReader import *

from StringInsert import StringInsert

# Class to handle the primitive drawing operations for rendering DVI
# The point of this particular class is that you might not want to draw *anything*,
# just read the .dvi file

class DviDevice: # subclassed by PysCmdDevice, PSPathDevice

	def __init__( self, prefersChars = False ):
		self.prefersChars = prefersChars

	def beginDocument( self, num, den, mag, reader ):
		pass
		
	def endDocument( self, reader ):
		pass

	def beginPage( self, reader ):
		pass
		
	def endPage( self, reader ):
		pass

	def startFont( self, font, scaleFactor, reader):
		pass
		
	def putString( self, string, reader):
		pass

	def putChar( self, h, v, c, reader):
		pass

	def putRule( self, h, v, a, b ):
		pass

	def doSpecial( self, dvc, reader ):
		pass

# Class that handles rendering of DVI to a device.  The idea is that a device should know
# next to nothing about DVI (and in particular the intricacies of virtual fonts); 
# it just renders what and where it is told to render.  On the
# other hand, DVIReader has multiple uses, not just rendering to a device (e.g. DVIType)
# So DviToDevice is an intermediate class that handles common code for rendering, but
# not device specific drawing.  Maybe this should be made smarter in the future about gathering
# together strings of a common font obtained through individual virtual font characters.

# a typical device is PysCmdDevice: turns the .dvi file to a command array
# init() reads the file, render() feeds it to the device

class DviToDevice(DviReader):
	def __init__(self, file, device ):
		DviReader.__init__(self, file)
		self.device = device
		self.currentString = StringInsert()
		self.lastSentFont = None
		self.lastSentFontSize = 0
		self.origin = None

	def render( self ):
		if self.device.prefersChars: # This is probably now obsolete
			self._renderWithChars()
		else:
			self._renderWithStrings()

	# This is probably obsolete.  Intended to handle character paths ... ?
	def _renderWithChars( self ):
		while (True):
			c = self.getCommand()
			# FIXME: WAC We are only building strings out of Set commands. We are doing *nothing*
			#        with the put commands.  Sheesh.
			if ( (0 <= c.index < 132 ) and (not self.currentFont.isVirtual) ):
				self.device.putChar( self.h, self.v, c.index, self )
				c.execute()
				# adjusts the bounding box
				i = c.getCharIndex()
				self.adjustbbox(i)
				if self.origin == None:
					# print "(a) setting origin" # FIXME: WAC
					self.origin = [ self.h, self.v ]
			else: # we are done reading the current string
				c.execute()
				if (0 <= c.index < 132 ):
					# We got here because of a virtual font.  Probably the current font changed.
					self.sendStartFont()
					if self.origin == None:
						# print "(b) setting origin" # FIXME: WAC
						self.origin = [ self.h, self.v ]
			if (c.index == self.EOF()):
				self.device.endDocument(self)
				break

	# in new version, we render *only* with strings,
	# making currentString a StringInsert, at least temporarily while debugging
	def _renderWithStrings( self ):
		while (True):
			c = self.getCommand()
			# print "[", str(c), "]"
			# print "[", c.index, "]"
			# FIXME: We are only building strings out of Set commands. We are doing *nothing*
			#        with the put commands.  Sheesh.
			if ( (0 <= c.index < 132 ) and (not self.currentFont.isVirtual) ):
				# If starting a string, cache the current values of h and v to feed to the device
				# as the start of string.  We do this *before* executing the command
				if self.currentString.size() == 0:
					self.h0 = self.h
					self.v0 = self.v # don't need?
				self.currentString.addMetric((self.h, self.v))
				c.execute()
				
				# adjusts the bounding box
				i = c.getCharIndex()
				self.adjustbbox(i)
				self.currentString.addChar(i)
			else: # we are done reading the current string
				if self.currentString.size() > 0:
					if self.origin == None:
						# print "current string =", "[" + str(self.currentString) + "]"
						self.origin = [ self.h0, self.v0 ]
					self.currentString.addMetric((self.h, self.v)) 
					# NO!  not necessarily the end of the current string FIX: WAC
					self.device.putString(self.currentString, self) 
				self.currentString = StringInsert()
				c.execute()
				if (0 <= c.index < 132 ):
					# We got here because of a virtual font.  Probably the current font changed.
					self.sendStartFont()
			if (c.index == self.EOF()):
				self.device.endDocument(self)
				break

	def sendStartFont( self ):
		if( not self.currentFont.isVirtual ):
			pointSize = self.scaleFactor*self.currentFont.scaledSize;
			if (self.currentFont.fontName != self.lastSentFont) or ( pointSize != self.lastSentFontSize):
				self.lastSentFont = self.currentFont.fontName; self.lastSentFontSize = pointSize
				self.device.startFont( self.currentFont, self.scaleFactor, self )

	def execPreAmble( self, dvc ):
		DviReader.execPreAmble(self, dvc)
		self.device.beginDocument( dvc.num, dvc.den, dvc.mag, self )

	def execSetRule(self, dvc):
		self.device.putRule( self.h, self.v, dvc.a*self.scaleFactor, dvc.b*self.scaleFactor, self )
		DviReader.execSetRule(self, dvc)

	def execPutRule(self, dvc):
		self.device.putRule( self.h, self.v, dvc.a*self.scaleFactor, dvc.b*self.scaleFactor, self)
		DviReader.execPutRule(self, dvc)

	def execFntNum(self, dvc):
		DviReader.execFntNum(self, dvc)
		self.sendStartFont();

	def execFnt(self, dvc):
		DviReader.execFnt(self, dvc,)
		self.sendStartFont()

	def execBop(self, dvc):
		DviReader.execBop(self, dvc)
		self.device.beginPage( self )
			
	def execEop(self, dvc):
		DviReader.execEop(self, dvc)
		self.device.endPage( self )

	def doSpecial(self, dvc):
		self.device.doSpecial( dvc, self )
		DviReader.doSpecial(self, dvc)


